package edu.gmu.cs;

import java.util.*;

/**
 * A graph class with an implementation of Graph's shortest distance algorithm
 */
public class Graph<T> {

    protected Set<T> vertices;
    protected Map<String, Integer> edges;

    public Graph(Set<T> vertices) {
        this.vertices = new HashSet<T>(vertices);
        this.edges = new HashMap<String, Integer>();
    }

    /**
     * Return a label used to refer to an edge.
     * @param src
     * @param dst
     * @return
     */
    public String edgeLabel(T src, T dst) {
        return String.format("%s:%s", src, dst);
    }

    public void addEdge(T src, T dst, Integer weight) {
        this.edges.put(edgeLabel(src, dst), weight);
    }

    /**
     * Check if an edge exists between src and dst.
     * @param src
     * @param dst
     * @return
     */
    public boolean hasEdge(T src, T dst) {
        return edges.containsKey(edgeLabel(src, dst));
    }

    /**
     * Return weight for the given edge.
     * @param src
     * @param dst
     * @return edge weight
     */
    public Integer getWeight(T src, T dst) {
        if (hasEdge(src, dst)) {
            throw new RuntimeException("No such edge exists.");
        }

        return edges.get(edgeLabel(src, dst));
    }

    /**
     * Return the shortest path between the given source and destination vertices.
     *
     * @return If a path is found, returns the path of nodes, in order from start to end.
     * If no path is found, returns null.
     */
    public List<T> findShortestPath(T start, T end) {
        Map<T, Integer> distance = new HashMap<T, Integer>();
        Map<T, T> predecessor = new HashMap<T, T>();
        Set<T> allowedVertices = new HashSet<T>();

        return null;
    }
}
